/*
 * GStreamer Mir video sink
 * Copyright (C) 2017 Canonical Ltd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 */

#ifndef __GST_MIR_VIDEO_SINK_H__
#define __GST_MIR_VIDEO_SINK_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <assert.h>
#include <unistd.h>
#include <stdint.h>

#define GL_GLEXT_PROTOTYPES
#define EGL_EGLEXT_PROTOTYPES

#include <mir_toolkit/mir_client_library.h>
#include <EGL/egl.h>
#include <EGL/eglext.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

#include <gst/gst.h>
#include <gst/video/video.h>
#include <gst/video/gstvideosink.h>
#include <gst/video/gstvideometa.h>

#define GST_TYPE_MIR_SINK \
        (gst_mir_sink_get_type())
#define GST_MIR_SINK(obj) \
        (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_MIR_SINK,GstMirSink))
#define GST_MIR_SINK_CLASS(klass) \
        (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_MIR_SINK,GstMirSinkClass))
#define GST_IS_MIR_SINK(obj) \
        (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_MIR_SINK))
#define GST_IS_MIR_SINK_CLASS(klass) \
        (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MIR_SINK))
#define GST_MIR_SINK_GET_CLASS(inst) \
        (G_TYPE_INSTANCE_GET_CLASS ((inst), GST_TYPE_MIR_SINK, GstMirSinkClass))

typedef struct _GstMirSink GstMirSink;
typedef struct _GstMirSinkClass GstMirSinkClass;

struct _GstMirSink
{
  GstVideoSink parent;

  GstBufferPool *pool;

  GMutex mir_lock;

  /* Whether to export buffers or render them in a local window */
  gboolean export_buffers;
  gint video_width;
  gint video_height;
  GstVideoFormat video_format;

  MirConnection *conn;
  MirWindow *window;

  EGLDisplay egldisplay;
  EGLSurface eglsurface;
  EGLContext eglctx;

  GLuint vshader, fshader;

  /* Indexes to shader variables */
  GLint attrpos;
  GLint attrcordYUV[3];
  GLint sampYUV[3];
  GLint yuvmat;

  GLuint text_id[3];

  GLuint prog;

  /* Framebuffer object */
  GLuint fbo;
  GLuint render_tex;

  int buf_fd;
};

struct _GstMirSinkClass
{
  GstVideoSinkClass parent;
};

GType gst_mir_sink_get_type (void) G_GNUC_CONST;

G_END_DECLS

#endif /* __GST_MIR_VIDEO_SINK_H__ */
